{% extends 'base.html' %}

{% block title %}Import web page{% endblock %}
{% block header %}Import web page{% endblock %}

{% block body %}

<p>This import is very primitive -- it grabs <i>all</i> the headings
and text from an HTML page.<br/>  This likely includes stuff you don't
want, so edit the textbox of the Book form when it is displayed!</p>

<br />

<form action="/book/new" method="get">
  <label for="importurl">Import URL:</label>
  <input type="text" id="importurl" name="importurl" class="form-text" style="width:75%;"><br><br>
  <input type="submit" id="import" value="Import">
</form>

{% endblock %}
