<!DOCTYPE html>
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8" />
    <title>Form</title>
    
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/jquery-ui.css') }}" />
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/tagify.css') }}" />
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/tagify_overrides.css') }}" />
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/styles.css') }}" />

    <link rel="stylesheet" type="text/css" href="/theme/current">
    <link rel="stylesheet" type="text/css" href="/theme/custom_styles">

    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery.js') }}" charset="utf-8"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery.scrollTo.min.js') }}" charset="utf-8"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery-ui.min.js') }}"  charset="utf-8"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/jquery.jeditable.mini.js') }}" charset="utf-8"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/tagify.min.js') }}" charset="utf-8"></script>
    <script type="text/javascript" src="{{ url_for('static', filename='js/tagify.polyfills.min.js') }}" charset="utf-8"></script>

    <script type="text/javascript">
      var temp_tags = '{{ tags | tojson }}';
      var TAGS = Array.from(JSON.parse(temp_tags));
    </script>

  </head>

  <body>
    {% if term.flash_message %}
    <p class='small-flash-notice'>{{ term.flash_message }}</p>
    {% endif %}
    {% include 'term/_form.html' %}
  </body>

</html>
