{% extends 'base.html' %}

{% block header %}
{% if term.language_id is none %}New Term{% else %}Edit Term{% endif %}
{% endblock %}

{% block body %}
<div id="term_form_left">
  {% include('term/_form.html') %}
  <a href="/term/index">Back to list</a>
</div>


<div id="term_form_right">
  <iframe name="dictframe" class="rightreadingframe" src="/read/empty" scrolling="auto"
          style="height: 100%; width: 100%;" frameBorder="1">
    Frame support required.
  </iframe>
</div>

{% endblock %}
